"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Iterator = void 0;
class Iterator {
    constructor() {
        this.description = {
            displayName: 'Iterator',
            name: 'iterator',
            group: ['transform'],
            icon: 'file:repeat.svg',
            version: 1,
            description: 'Iterator',
            defaults: {
                name: 'Iterator',
            },
            inputs: ['main'],
            outputs: ['main', 'main'],
            outputNames: ['Done', 'Iterate'],
            properties: [
                {
                    displayName: 'Iterator Type',
                    name: 'iterType',
                    type: 'options',
                    options: [
                        {
                            name: 'Auto Increment',
                            value: 'increment',
                        },
                        {
                            name: 'Reference Next',
                            value: 'nextRef',
                        },
                    ],
                    default: 'increment',
                    description: 'Type of Iterator',
                },
                {
                    displayName: 'Increment Start',
                    name: 'incrementStart',
                    type: 'number',
                    default: 0,
                    description: 'Start of the auto increment',
                    displayOptions: {
                        show: {
                            iterType: ['increment'],
                        },
                    },
                },
                {
                    displayName: 'Increment Size',
                    name: 'incrementSize',
                    type: 'number',
                    default: 1,
                    description: 'Size of the auto increment',
                    displayOptions: {
                        show: {
                            iterType: ['increment'],
                        },
                    },
                },
                {
                    displayName: 'Reference Start',
                    name: 'refStart',
                    type: 'string',
                    default: '',
                    description: 'Value to start with',
                    displayOptions: {
                        show: {
                            iterType: ['nextRef'],
                        },
                    },
                },
                {
                    displayName: 'Reference Next',
                    name: 'refNext',
                    type: 'string',
                    default: '',
                    description: 'Value of the next reference, for example the next page',
                    displayOptions: {
                        show: {
                            iterType: ['nextRef'],
                        },
                    },
                },
                {
                    displayName: 'Options',
                    name: 'options',
                    type: 'collection',
                    placeholder: 'Add Option',
                    default: {},
                    options: [
                        {
                            displayName: 'Expected Item Count',
                            name: 'expectedItemCount',
                            type: 'number',
                            typeOptions: {
                                minValue: 1,
                            },
                            default: 50,
                            description: 'Expected item count, is used to determine if the loop should continue. When using an offset this is often the same as the Increment Size.',
                        },
                        {
                            displayName: 'Is There Another Page',
                            name: 'anotherPage',
                            type: 'boolean',
                            default: false,
                            description: 'Whether there is another page, this can be set to a field returned by an API for example',
                        },
                        {
                            displayName: 'Combine',
                            name: 'combine',
                            type: 'boolean',
                            default: false,
                            description: 'Whether the node will combine all items before outputting to Done',
                        },
                        {
                            displayName: 'Limit',
                            name: 'limit',
                            type: 'number',
                            typeOptions: {
                                minValue: 1,
                            },
                            default: 50,
                            description: 'Max number of results to return',
                        },
                        {
                            displayName: 'Iteration Limit',
                            name: 'iterationLimit',
                            type: 'number',
                            typeOptions: {
                                minValue: 1,
                            },
                            default: 50,
                            description: 'Max number of iterations',
                        },
                    ],
                },
            ],
        };
    }
    async execute() {
        const items = this.getInputData();
        const nodeContext = this.getContext('node');
        const iterType = this.getNodeParameter('iterType', 0, '');
        const limit = this.getNodeParameter('options.limit', 0, 0);
        const expectedItemCount = this.getNodeParameter('options.expectedItemCount', 0, 0);
        const iterationLimit = this.getNodeParameter('options.iterationLimit', 0, 0);
        const combine = this.getNodeParameter('options.combine', 0, '');
        const anotherPage = this.getNodeParameter('options.anotherPage', 0, true);
        let done = !anotherPage;
        const returnItems = [[], []];
        if (iterType === 'increment') {
            if (nodeContext.currentIncrement === undefined) {
                const incrementStart = this.getNodeParameter('incrementStart', 0, 0);
                nodeContext.currentIncrement = incrementStart;
                nodeContext.processedItems = [];
                nodeContext.nrOfIterations = 0;
                done = false;
            }
            else {
                const incrementSize = this.getNodeParameter('incrementSize', 0, 1);
                nodeContext.currentIncrement = nodeContext.currentIncrement + incrementSize;
                nodeContext.nrOfIterations++;
                if (combine === true) {
                    nodeContext.processedItems.push.apply(nodeContext.processedItems, items);
                }
                else {
                    nodeContext.processedItems = items;
                }
                if (expectedItemCount !== 0 && items.length < expectedItemCount) {
                    done = true;
                }
            }
        }
        if (iterType === 'nextRef') {
            if (nodeContext.currentReference === undefined) {
                const refStart = this.getNodeParameter('refStart', 0, '');
                nodeContext.currentReference = refStart;
                nodeContext.processedItems = [];
                nodeContext.nrOfIterations = 0;
                done = false;
            }
            else {
                const refNext = this.getNodeParameter('refNext', 0, '');
                nodeContext.currentReference = refNext;
                nodeContext.nrOfIterations++;
                if (combine === true) {
                    nodeContext.processedItems.push.apply(nodeContext.processedItems, items);
                }
                else {
                    nodeContext.processedItems = items;
                }
                if (expectedItemCount !== 0 && items.length < expectedItemCount) {
                    done = true;
                }
            }
            if (nodeContext.currentReference === null) {
                done = true;
            }
        }
        if (nodeContext.nrOfIterations >= iterationLimit) {
            console.log(iterationLimit);
            done = true;
        }
        if (limit !== 0 && nodeContext.processedItems.length >= limit) {
            returnItems[0] = nodeContext.processedItems.slice(0, limit);
        }
        else if (done === true) {
            returnItems[0] = nodeContext.processedItems.filter((x) => Object.keys(x.json).length !== 0);
        }
        else {
            returnItems[1] = [items[items.length - 1]];
        }
        return returnItems;
    }
}
exports.Iterator = Iterator;
//# sourceMappingURL=Iterator.node.js.map